"""
DB 와 연결하는 부분
"""

from logging import getLogger

import pandas as pd

from pycontainerutils.db.DB_Adapter import DBAdapter

logger = getLogger(__name__)


class DBAdapterDF(DBAdapter):
    """
    데이터를 사용하는 엔진
    해당 데이터를 다루는 모든 조작을 담당
    """

    def check_delete_insert_df(self, table_name, df, primary_keys):
        """DB에 중복된 데이터는 모두 제거후 새 데이터 삽입
        :param table_name: 테이블명
        :param df: 새로 넣을 dataframe
        :param primary_keys: 중복확인시 비교할 값
        :return:
        """
        for x, item in df.iterrows():
            # DB 데이터 삭제
            delete_sql = f"DELETE FROM {table_name}"
            delete_sql = self.make_sql_by_df_keys(item, delete_sql, primary_keys)
            self.execute_sql(delete_sql)

            # 데이터 삽입
            self.insert_dict(table_name, item.to_dict())

    @staticmethod
    def make_sql_by_df_keys(data, init_sql, primary_keys):
        # DB 데이터 조회

        where_key_list = []
        for key in primary_keys:
            # 정수나 실수면 그냥 입력
            if type(data[key]) == int or type(data[key]) == float:
                where_key_list.append(f" {key} = {data[key]}")
            # 문자열인 경우 '' 추가
            else:
                where_key_list.append(f" {key} = '{data[key]}'")

        # list를 합쳐서 sql문 제작
        result_sql = init_sql + " WHERE " + ' AND'.join(where_key_list)
        return result_sql



import json
import logging.config

from pycontainerutils.logger.base_logger_config import base_logger_config
from pycontainerutils.logger.db.DB_handler import DatabaseHandler


class Log_settings:
    log_config = None

    def initialize(self):
        logging.config.dictConfig(self.log_config)

    @classmethod
    def logger_settings(cls, logger_config, container_name, db_info=None):
        """로그 설정
        :param db_info: 로그를 저장할 db
        :param container_name: db log에 들어갈 컨테이너 이름
        :param logger_config: logger 설정
        :return:
        """
        # db 연결정보 입력시 db handler 설정
        if db_info is not None:
            DatabaseHandler.init_db_adapter(container=container_name, db_info=db_info)
        # 기본 정의된 handler 사용
        cls.log_config = base_logger_config
        # logger만 입력받아서 사용
        cls.log_config['loggers'] = logger_config
        # 생성
        logging.config.dictConfig(cls.log_config)


def open_log_setting_py(log_config):
    logging.config.dictConfig(log_config.log_config)


def open_log_setting_json():
    """
    json 파일 형식으로 된 logger 설정파일을 읽어와 로거를 설정한다.
    :return:
    """
    with open('loggers.json') as f:
        config_json = json.load(f)
        print("로그 설정")
        # print(config)
        logging.config.dictConfig(config_json)



import unittest

from pycontainerutils.db.BaseDB_Adapter import BaseDBAdapter
from pycontainerutils.db.DB_Adapter import DBAdapter

BaseDBAdapter.init_settings(
    databases={
        'test': {
            "database": 'pycontainerutils',
            "user": 'pycontainerutils',
            "password": 'pycontainerutils',
            "host": "127.0.0.1",
            "port": 12345,
        },
    },
    default_db_name="test",
    is_echo_sql=True
)


class DBAdapterTest(unittest.TestCase):
    test_table = f"test_table"
    db = DBAdapter(name="test DBAdapterTest")
    # table sql
    create_table_sql = f"CREATE TABLE IF NOT EXISTS {test_table} (" \
                       f" id     serial not null    primary key, " \
                       f" val_int   integer, " \
                       f" val_real   real," \
                       f" val_str varchar(50)" \
                       f");"
    delete_table_sql = f"DROP TABLE {test_table}"
    # test data
    test_sample_data = [123, 0.3, 'test str 1']
    # data sql
    insert_sql = f"INSERT INTO {test_table} (val_int, val_real, val_str)" \
                 f"VALUES ({test_sample_data[0]}, {test_sample_data[1]}, '{test_sample_data[2]}');"
    select_sql = f"SELECT * FROM {test_table}"
    delete_sql = f"DELETE FROM {test_table}"

    def setUp(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        self.db.execute_sql(self.create_table_sql)

        pass

    def tearDown(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        self.db.execute_sql(self.delete_table_sql)

        pass

    def test_connect(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        db = DBAdapter(name="test_connect DBAdapterTest")
        db.execute_sql(self.select_sql)
        pass

    def test_crud(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        db = DBAdapter(name="test_crud DBAdapterTest")
        # insert
        db.execute_sql(self.insert_sql)
        # check data insert - 정상적으로 데이터가 추가되었는지 확인
        data = self.db.fetch_data_by_sql(self.select_sql)
        self.assertEqual(data, [(1, *self.test_sample_data)])

        # 데이터 삭제
        self.db.execute_sql(self.delete_sql)

        # 데이터 존재 확인 - 데이터가 없어야 함
        data = self.db.fetch_data_by_sql(self.select_sql)
        self.assertEqual(data, [])

    def test_insert_dict(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        db = DBAdapter(name="test_crud DBAdapterTest")
        # insert
        test_dict = {
            "val_int":  self.test_sample_data[0],
            "val_real":  self.test_sample_data[1],
            "val_str": self.test_sample_data[2],
        }
        db.insert_dict(self.test_table, test_dict)
        # check data insert - 정상적으로 데이터가 추가되었는지 확인
        data = self.db.fetch_data_by_sql(self.select_sql)
        self.assertEqual(data, [(1, *self.test_sample_data)])
        #
        # # 데이터 삭제
        # self.db.execute_sql(self.delete_sql)
        #
        # # 데이터 존재 확인 - 데이터가 없어야 함
        # data = self.db.fetch_data_by_sql(self.select_sql)
        # self.assertEqual(data, [])



import unittest
from datetime import datetime, timedelta

from pandas.core.frame import DataFrame

from pycontainerutils.db.BaseDB_Adapter import BaseDBAdapter
from pycontainerutils.db.df_utils import DBAdapterDF

BaseDBAdapter.init_settings(
    databases={
        'test': {
            "database": 'pycontainerutils',
            "user": 'pycontainerutils',
            "password": 'pycontainerutils',
            "host": "127.0.0.1",
            "port": 12345,
        },
    },
    default_db_name="test",
    is_echo_sql=False
)


class DBAdapterDFTest(unittest.TestCase):
    test_table = f"test_table"
    db = DBAdapterDF(name="test DBAdapterDFTest")
    # table sql
    create_table_sql = f"CREATE TABLE IF NOT EXISTS {test_table} (" \
                       f" id     serial not null    primary key, " \
                       f" val_int   integer, " \
                       f" val_real   real," \
                       f" val_str varchar(50)" \
                       f");"
    delete_table_sql = f"DROP TABLE {test_table}"
    # test data
    test_sample_data = [123, 0.3, 'test str 1']
    # data sql
    insert_sql = f"INSERT INTO {test_table} (val_int, val_real, val_str)" \
                 f"VALUES ({test_sample_data[0]}, {test_sample_data[1]}, '{test_sample_data[2]}');"
    select_sql = f"SELECT * FROM {test_table}"
    delete_sql = f"DELETE FROM {test_table}"

    def setUp(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        self.db.execute_sql(self.create_table_sql)

        pass

    def tearDown(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        self.db.execute_sql(self.delete_table_sql)

        pass

    def test_connect(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        db = DBAdapterDF(name="test_connect DBAdapterDFTest")
        db.execute_sql(self.select_sql)
        pass

    def test_crud(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        db = DBAdapterDF(name="test_crud DBAdapterDFTest")
        # insert
        db.execute_sql(self.insert_sql)
        # check data insert - 정상적으로 데이터가 추가되었는지 확인
        data = self.db.fetch_data_by_sql(self.select_sql)
        self.assertEqual(data, [(1, *self.test_sample_data)])

        # 데이터 삭제
        self.db.execute_sql(self.delete_sql)

        # 데이터 존재 확인 - 데이터가 없어야 함
        data = self.db.fetch_data_by_sql(self.select_sql)
        self.assertEqual(data, [])

    def test_check_delete_insert_df(self):
        db = DBAdapterDF(name="test_check_delete_insert_df DBAdapterDFTest")
        # 테이블 생성
        table_create_sql = f"CREATE TABLE IF NOT EXISTS test_df (" \
                           "str1                       varchar(50)      not null, " \
                           "str2                       varchar(50)      not null, " \
                           "time_start                    timestamp        not null, " \
                           "time_end                     timestamp        not null, " \
                           "val_int                   integer          not null, " \
                           "val_float double precision not null " \
                           ");"
        db.execute_sql(sql=table_create_sql)
        df = DataFrame(data={
            "str1": ["test1", "test2"],
            "str2": ["test10", "test22"],
            "time_start": [datetime(year=2020, month=1, day=1), datetime(year=2020, month=3, day=1)],
            "time_end": [datetime(year=2020, month=6, day=30), datetime(year=2020, month=7, day=10)],
            "val_int": [10, 4],
            "val_float": [10.3, 4.3],
        })
        db.check_delete_insert_df(table_name="test_df", df=df, primary_keys=["str1", "str2", "time_start", "time_end"])

        # 테이블 삭제
        # table_delete_sql = f"DROP TABLE test_df"
        # db.execute_sql(sql=table_delete_sql)




import unittest
from logging import getLogger

from pycontainerutils.logger.setting_logger import Log_settings

Log_settings.logger_settings(
    logger_config={
        "": {
            "level": "DEBUG",
            "handlers": ["console"],
        },
        "test_logger_console": {
            "level": "INFO",
            "handlers": ["console"],
            # "propagate": True,
            "propagate": False,
        },
    },
    container_name="LoggerConsoleTest",
)

logger = getLogger(__name__)


class LoggerConsoleTest(unittest.TestCase):

    def setUp(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        pass

    def tearDown(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        pass

    def test_connect(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        logger.debug("hello debug")
        logger.info("hello info")
        logger.warning("hello warning")
        logger.error("hello error")
        pass



import setuptools

import unittest


def my_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='test_*.py')
    return test_suite


setuptools.setup(
    name="pycontainerutils",  # 패키지 명
    version='0.0.1.1',
    license='MIT',
    author='greenrain',
    author_email='kdwkdw0078@gmail.com',
    description='docker container utilities',
    long_description=open('README.md', encoding='UTF-8').read(),
    url="https://github.com/greenrain78",
    packages=setuptools.find_packages(),
    # test_suite='setup.my_test_suite',
    install_requires=[
        "coloredlogs>=15.0.1",
        "numpy>=1.21.4",
        "pandas>=1.3.4",
        "psycopg2>=2.9.2",
        "SQLAlchemy>=1.4.27",
        "APScheduler>=3.8.1",
    ]
)



"""
Schedule DB SchedulerManager
실질적인 스캐줄 객체
"""
import functools
import logging

from apscheduler.schedulers.background import BackgroundScheduler

logger = logging.getLogger(__name__)


class MainScheduler:

    def __init__(self, name: str = "default"):
        # 스캐줄 생성
        self.name = name
        self.schedule = BackgroundScheduler()
        logger.info(f"{self.name} 스케줄러 Manager init")

    def create_job(self, func, trigger: str = "cron", schedule_id: str = None, **schedule_time):
        """
        반복적으로 실행할 작업 생성
        :param func: 실행할 함수
        :param trigger: 스케줄러 트리거 (cron, interval)
        :param schedule_id: str
        :param schedule_time: dict 형식의 시간 인자 - cron 참고
        :return:
        """
        # id 미기입시
        if schedule_id is None:
            id = f"{func.__module__}_{func.__name__}"
            self.schedule.add_job(func, trigger, id=id, **schedule_time)
            logger.info(f"{self.name} 스케줄러 create_job : {id} - {trigger}({schedule_time})")
        else:
            self.schedule.add_job(func, trigger, id=schedule_id, **schedule_time)
            logger.info(f"{self.name} 스케줄러 create_job : {schedule_id} - {trigger}({schedule_time})")

    def delete_job(self, schedule_id: str):
        """
        스케줄에 등록되어 있는 작업을 제거
        :param schedule_id: 스케줄 id
        :return:
        """
        self.schedule.resume_job(schedule_id)
        logger.info(f"{self.name} 스케줄러 delete_job: {schedule_id}")

    def run(self):
        self.schedule.start()
        logger.info(f"{self.name} 스케줄러 Run")
#
#
# # args 사용 불가 - 중요!
# def add_schedule(func=None, **decorator_base_kwargs):
#     # 속성을 미기입 - > decorator 호출
#     if func:
#         print(f"add_schedule func - {type(func)}", flush=True)
#         return ScheduleDecorators(func, **decorator_base_kwargs)
#     # 속성을 기입 - > wrapper 로 감싼후 decorator 호출
#     else:
#         # 아무 인자도 들어오지 않음
#         def base_wrapper(function):
#             # 이중 Decorator 해결
#             wrapped = ScheduleDecorators(function, **decorator_base_kwargs)()
#             return wrapped
#
#         return base_wrapper
#
#
# # 실행 시 에만 동작
# class ScheduleDecorators(object):
#     scheduler = MainScheduler('decorators scheduler')
#
#     # Decorator 생성
#     # args 사용 금지 - 불가능
#     def __init__(self, func, **init_kwargs):
#         self.func = func
#         self.args = init_kwargs
#
#     # Decorator 호출
#     def __call__(self):
#         decorator_self = self
#         logger.info(f"scheduler({self.scheduler.name}) add job {self.func.__name__} - {self.args}")
#
#         # 스캐줄러 등록
#         self.scheduler.create_job(self.func, **self.args)
#
#         # 함수 Decorator 실행
#         def wrapper(*args, **kwargs):
#             function = self.func(*args, **kwargs)
#             return function
#
#         return wrapper

#
# # 실행 시 에만 동작
# class ScheduleDecorators(object):
#     scheduler = MainScheduler('decorators scheduler')
#
#     # Decorator 생성
#     # args 사용 금지 - 불가능
#     def __init__(self, **init_kwargs):
#         self.kargs = init_kwargs
#         # self.scheduler.create_job()
#
#     def __call__(self, fn):
#         @functools.wraps(fn)
#         def decorated(*args, **kwargs):
#             print()
#             print("In my decorator before call, with arg %s" % self.arg)
#             result = fn(*args, **kwargs)
#             print("In my decorator after call, with arg %s" % self.arg)
#             return result
#
#         return decorated
#
#     # # Decorator 호출
#     # def __call__(self):
#     #     logger.debug(f"scheduler({self.scheduler.name}) add job {self.func.__name__} - {self.args}")
#     #
#     #     # 스캐줄러 등록
#     #     # self.scheduler.create_job_method(self.func, self.func.__name__, self.scheduler, **self.args)
#     #
#     #     # 함수 Decorator 실행
#     #     def wrapper(*args, **kwargs):
#     #         function = self.func(*args, **kwargs)
#     #         return function
#     #
#     #     return wrapper
#
#
# class MyDecorator(object):
#     def __init__(self, **kwargs):
#         self.kwargs = kwargs
#         print(kwargs)
#
#     def __call__(self, fn):
#         @functools.wraps(fn)
#         def decorated(*args, **kwargs):
#             print()
#             print("In my decorator before call, with arg %s" % self.arg)
#             result = fn(*args, **kwargs)
#             print("In my decorator after call, with arg %s" % self.arg)
#             return result
#
#         return decorated
# #
# # # 모듈 수준의 스캐줄러 생성
# # module_scheduler = MainScheduler('module_scheduler')
# #
# #
# # def schedule_start():
# #     module_scheduler.run()
# #
# #
# # # args 사용 불가 - 중요!
# # def add_schedule(func=None, **decorator_base_kwargs):
# #     # 속성을 미기입 - > decorator 호출
# #     if func:
# #         print(f"add_schedule func - {type(func)}", flush=True)
# #         return schedule_decorators(func, **decorator_base_kwargs)
# #     # 속성을 기입 - > wrapper 로 감싼후 decorator 호출
# #     else:
# #         # 아무 인자도 들어오지 않음
# #         def base_wrapper(function):
# #             # 이중 Decorator 해결
# #             wrapped = schedule_decorators(function, **decorator_base_kwargs)()
# #             return wrapped
# #
# #         return base_wrapper
# #
